@@ -120,7 +120,10 @@ int run_tests_from_file(const char * filename) {
 		return EXIT_FAILURE;
 	}
 	fstat(fd, &sample_stat);
-	sample_buffer = mmap(0, sample_stat.st_size, PROT_READ, MAP_PRIVATE, fd, 0);
+
+	sample_buffer = malloc(sample_stat.st_size + 1);
+	read(fd, sample_buffer, sample_stat.st_size);
+	sample_buffer[sample_stat.st_size] = 0;
 
 	if (sample_buffer == (void *) -1)
 	{
@@ -162,7 +165,7 @@ int run_tests_from_file(const char * filename) {
 
 	pg_query_free_split_result(split_result);
 
-	munmap(sample_buffer, sample_stat.st_size);
+	free(sample_buffer);
 	close(fd);
 
 	return ret_code;
@@ -192,8 +195,11 @@ const char* regressFilenames[] = {
 	"collate.icu.utf8.sql",
 	"collate.linux.utf8.sql",
 	"collate.sql",
+	"collate.windows.win1252.sql",
 	"combocid.sql",
 	"comments.sql",
+	"compression.sql",
+	"constraints.sql",
 	"conversion.sql",
 	"copy.sql",
 	"copy2.sql",
@@ -204,14 +210,15 @@ const char* regressFilenames[] = {
 	"create_cast.sql",
 	"create_function_c.sql",
 	"create_function_sql.sql",
-	"create_index_spgist.sql",
 	"create_index.sql",
+	"create_index_spgist.sql",
 	"create_misc.sql",
 	"create_operator.sql",
-	"create_role.sql",
 	"create_procedure.sql",
-	"create_table_like.sql",
+	"create_role.sql",
+	"create_schema.sql",
 	"create_table.sql",
+	"create_table_like.sql",
 	"create_type.sql",
 	"create_view.sql",
 	"date.sql",
@@ -245,28 +252,28 @@ const char* regressFilenames[] = {
 	"horology.sql",
 	"identity.sql",
 	"incremental_sort.sql",
-	"index_including_gist.sql",
 	"index_including.sql",
+	"index_including_gist.sql",
 	"indexing.sql",
 	"indirect_toast.sql",
 	"inet.sql",
 	"infinite_recurse.sql",
 	"inherit.sql",
 	"init_privs.sql",
-	"insert_conflict.sql",
 	"insert.sql",
+	"insert_conflict.sql",
 	"int2.sql",
 	"int4.sql",
 	"int8.sql",
 	"interval.sql",
-	"join_hash.sql",
 	"join.sql",
-	"json_encoding.sql",
+	"join_hash.sql",
 	"json.sql",
-	"jsonb_jsonpath.sql",
+	"json_encoding.sql",
 	"jsonb.sql",
-	"jsonpath_encoding.sql",
+	"jsonb_jsonpath.sql",
 	"jsonpath.sql",
+	"jsonpath_encoding.sql",
 	"largeobject.sql",
 	"limit.sql",
 	"line.sql",
@@ -275,16 +282,19 @@ const char* regressFilenames[] = {
 	"macaddr.sql",
 	"macaddr8.sql",
 	"matview.sql",
+	"md5.sql",
 	"memoize.sql",
 	"merge.sql",
 	"misc.sql",
 	"misc_functions.sql",
 	"misc_sanity.sql",
 	"money.sql",
+	"multirangetypes.sql",
+	"mvcc.sql",
 	"name.sql",
 	"namespace.sql",
-	"numeric_big.sql",
 	"numeric.sql",
+	"numeric_big.sql",
 	"object_address.sql",
 	"oid.sql",
 	"oidjoins.sql",
@@ -301,13 +311,13 @@ const char* regressFilenames[] = {
 	"point.sql",
 	"polygon.sql",
 	"polymorphism.sql",
-	"portals_p2.sql",
 	"portals.sql",
+	"portals_p2.sql",
 	"prepare.sql",
 	"prepared_xacts.sql",
 	"privileges.sql",
-	"psql_crosstab.sql",
 	"psql.sql",
+	"psql_crosstab.sql",
 	"publication.sql",
 	"random.sql",
 	"rangefuncs.sql",
@@ -324,18 +334,19 @@ const char* regressFilenames[] = {
 	"rules.sql",
 	"sanity_check.sql",
 	"security_label.sql",
-	"select_distinct_on.sql",
+	"select.sql",
 	"select_distinct.sql",
+	"select_distinct_on.sql",
 	"select_having.sql",
 	"select_implicit.sql",
 	"select_into.sql",
 	"select_parallel.sql",
 	"select_views.sql",
-	"select.sql",
 	"sequence.sql",
 	"spgist.sql",
-	"stats_ext.sql",
+	"sqljson.sql",
 	"stats.sql",
+	"stats_ext.sql",
 	"strings.sql",
 	"subscription.sql",
 	"subselect.sql",
@@ -369,6 +380,7 @@ const char* regressFilenames[] = {
 	"update.sql",
 	"uuid.sql",
 	"vacuum.sql",
+	"vacuum_parallel.sql",
 	"varchar.sql",
 	"window.sql",
 	"with.sql",
@@ -377,7 +389,7 @@ const char* regressFilenames[] = {
 	"xml.sql",
 	"xmlmap.sql"
 };
-size_t regressFilenameCount = 207;
+size_t regressFilenameCount = sizeof(regressFilenames) / sizeof(regressFilenames[0]);
 
 const char* plpgsqlRegressFilenames[] =
 {
@@ -393,7 +405,7 @@ const char* plpgsqlRegressFilenames[] =
 	"plpgsql_trigger.sql",
 	"plpgsql_varprops.sql"
 };
-size_t plpgsqlRegressFilenameCount = 11;
+size_t plpgsqlRegressFilenameCount = sizeof(plpgsqlRegressFilenames) / sizeof(plpgsqlRegressFilenames[0]);
 
 int main() {
 	size_t i;
